from types import SimpleNamespace
import requests
import json


url = "https://api.mnnai.ru"
version = "5.1.0"


class ServerError(Exception):
    def send_data(self, data):
        raise ServerError("")

class VersionNotFoundError(Exception):
    def send_data(self, data):
        raise VersionNotFoundError("")

def change(d):
    if isinstance(d, dict):
        return SimpleNamespace(**{k: change(v) for k, v in d.items()})
    elif isinstance(d, list):
        return [change(i) for i in d]
    else:
        return d

def GetModels():
    response = requests.get(f"{url}/models").json()
    return json.dumps(response, indent=4, ensure_ascii=False)

def get_pypi_version():
    try:
        response = requests.get(f"https://pypi.org/pypi/mnnai/json").json()
        if response["info"]["version"] > version:
            print(f"New mnnai version: {response['info']['version']} (current: {version}) | pip install -U mnnai")
    except requests.RequestException as e:
        raise VersionNotFoundError(f"Failed to get PyPI version: {e}")

from mnnai.Generator import MNN

get_pypi_version()



from mnnai import ServerError, change
from mnnai import url
import requests
import json

def Image(data):
    try:
        timeout = data["timeout"]
        headers = {
            "Content-Type": "application/json",
            "Authorization": data["key"]
        }
        payload = {
            "prompt": data["prompt"],
            "model": data["model"]
        }

        if data["debug"]:
            print(f"Sending a request to {url}/v1/images/generations")

        response = requests.post(f"{url}/v1/images/generations", headers=headers, json=payload, timeout=timeout)
        return change(response.json())

    except:
        raise ServerError("Unexpected error :(")


def Text(data):
    try:
        headers = {
            "Authorization": data["key"]
        }
        payload = {
            "model": data["model"],
            "messages": data["messages"],
            "stream": data["stream"],
            "web_search": data["web_search"]
        }

        if data["debug"]:
            print(f"Sending a request to {url}/v1/chat/completions")

        response = requests.post(f"{url}/v1/chat/completions", headers=headers, json=payload)

        res = []
        if data["stream"]:
            for token in response.text.split('\n'):
                if token:
                    token = token[5:]
                    if token != " [DONE]":
                        token = json.loads(token)
                        if "delta" in token["choices"][0]:
                            res.append(change(token))
            return res

        else:
            return change(response.json())
    except:
        raise ServerError("Unexpected error :(")



from mnnai import ServerError, GetModels
from mnnai import SyncAI, AsyncAI
from datetime import datetime


def valid(messages):
    if not isinstance(messages, list):
        return False

    for message in messages:
        if not isinstance(message, dict):
            return False

        if not all(key in message for key in ["role", "content"]):
            return False

        if not isinstance(message["role"], str) or message["role"] not in ["user", "assistant", "system"]:
            return False

        if not isinstance(message["content"], str):
            return False

    return True


def check_input(messages, model):
    if not messages:
        raise ValueError("The 'prompt' parameter must be filled in.")
    if not valid(messages):
        raise ValueError("Incorrect messages")
    if not model:
        raise ValueError("The 'model' parameter must be filled in.")

class MNN:
    def __init__(self, key: str = '', max_retries: int = 0, timeout: float = 600, debug: bool = False):
        self.key = key
        self.max_retries = max_retries
        self.timeout = timeout
        self.debug = debug
        self.chat = Chat(key, max_retries, timeout, debug)
        self.images = Images(key, max_retries, timeout, debug)

    def GetModels(self):
        return GetModels()


class Images:
    def __init__(self, key, max_retries, timeout, debug):
        self.key = key
        self.max_retries = max_retries
        self.timeout = timeout
        self.debug = debug

    def create(self, prompt: '', model: ''):
        start_time = datetime.now()
        if self.debug:
            print("Analyzing information entered by the user")

        if not prompt:
            raise ValueError("The 'prompt' parameter must be filled in.")
        if not model:
            raise ValueError("The 'model' parameter must be filled in.")

        data = {
            'prompt': prompt,
            'model': model,
            'key': self.key,
            'max_retries': self.max_retries,
            'timeout': self.timeout,
            'debug': self.debug
        }
        attempts = 0
        while attempts < self.max_retries + 1:
            if attempts >= 1:
                print(f"Attempt {attempts+1}")
            image = SyncAI.Image(data=data)

            if getattr(image, 'Error', None):
                if image.Error != 'Sorry, none of the providers responded, please use a different model':
                    raise ServerError(image.Error)
                attempts += 1
            else:
                end_time = datetime.now()
                time = end_time - start_time
                setattr(image.data[0].time, "total time", str(time))
                return image
        raise ServerError('Sorry, none of the providers responded, please use a different model')

    async def async_create(self, prompt: '', model: ''):
        start_time = datetime.now()
        if self.debug:
            print("Analyzing information entered by the user")

        if not prompt:
            raise ValueError("The 'prompt' parameter must be filled in.")
        if not model:
            raise ValueError("The 'model' parameter must be filled in.")

        data = {
            'prompt': prompt,
            'model': model,
            'key': self.key,
            'max_retries': self.max_retries,
            'timeout': self.timeout,
            'debug': self.debug
        }
        attempts = 0
        while attempts < self.max_retries + 1:
            if attempts >= 1:
                print(f"Attempt {attempts+1}")
            image = await AsyncAI.Image(data=data)

            if getattr(image, 'Error', None):
                if image.Error != 'Sorry, none of the providers responded, please use a different model':
                    raise ServerError(image.Error)
                attempts += 1
            else:
                end_time = datetime.now()
                time = end_time - start_time
                setattr(image.data[0].time, "total time", str(time))
                return image
        raise ServerError('Sorry, none of the providers responded, please use a different model')


class Chat:
    def __init__(self, key, max_retries, timeout, debug):
        self.key = key
        self.max_retries = max_retries
        self.timeout = timeout
        self.debug = debug

    def create(self, messages: list, model: str = '', stream: bool = False, web_search: bool = False):
        if self.debug:
            print("Analyzing information entered by the user")

        check_input(messages, model)

        data = {
            'messages': messages,
            'model': model,
            'key': self.key,
            'max_retries': self.max_retries,
            'timeout': self.timeout,
            'stream': stream,
            'debug': self.debug,
            'web_search': web_search
        }

        attempts = 0
        while attempts < self.max_retries + 1:
            if attempts >= 1:
                if self.debug:
                    print(f"Attempt {attempts + 1}")

            text = SyncAI.Text(data)

            if getattr(text, 'Error', None):
                if text.Error != 'Sorry, none of the providers responded, please use a different model':
                    raise ServerError(text.Error)
                attempts += 1
            else:
                return text

        raise ServerError('Sorry, none of the providers responded, please use a different model')

    async def async_create(self, messages: list, model: str = '', stream: bool = False, web_search: bool = False):
        if self.debug:
            print("Analyzing information entered by the user")

        check_input(messages, model)

        data = {
            'messages': messages,
            'model': model,
            'key': self.key,
            'max_retries': self.max_retries,
            'timeout': self.timeout,
            'stream': stream,
            'debug': self.debug,
            'web_search': web_search
        }

        attempts = 0
        while attempts < self.max_retries + 1:
            if attempts >= 1:
                if self.debug:
                    print(f"Attempt {attempts + 1}")
            if stream:
                async def async_generator():
                    async for token in AsyncAI.StreamText(data):
                        yield token

                return async_generator()

            else:
                text = await AsyncAI.Text(data)

                if getattr(text, 'Error', None):
                    if text.Error != 'Sorry, none of the providers responded, please use a different model':
                        raise ServerError(text.Error)
                    attempts += 1
                else:
                    return text

        raise ServerError('Sorry, none of the providers responded, please use a different model')


from mnnai import ServerError, change
from mnnai import url
import aiohttp
import json


async def Image(data):
    try:
        timeout = data["timeout"]
        headers = {
            "Content-Type": "application/json",
            "Authorization": data["key"]
        }
        payload = {
            "prompt": data["prompt"],
            "model": data["model"]
        }

        if data["debug"]:
            print(f"Sending a request to {url}/v1/images/generations")

        async with aiohttp.ClientSession() as session:
            async with session.post(f"{url}/v1/images/generations", headers=headers, json=payload, timeout=timeout) as response:
                return change(await response.json())
    except Exception:
        raise ServerError("Unexpected error :(")


async def Text(data):
    try:
        headers = {
            "Authorization": data["key"]
        }
        payload = {
            "model": data["model"],
            "messages": data["messages"],
            "stream": False,
            "web_search": data["web_search"]
        }

        if data["debug"]:
            print(f"Sending a request to {url}/v1/chat/completions")

        async with aiohttp.ClientSession() as session:
            async with session.post(f"{url}/v1/chat/completions", headers=headers, json=payload) as response:
                return change(json.loads(await response.text()))
    except Exception:
        raise ServerError("Unexpected error :(")


async def StreamText(data):
    try:
        headers = {
            "Authorization": data["key"]
        }
        payload = {
            "model": data["model"],
            "messages": data["messages"],
            "stream": True,
            "web_search": data["web_search"]
        }

        if data["debug"]:
            print(f"Sending a request to {url}/v1/chat/completions")

        async with aiohttp.ClientSession() as session:
            async with session.post(f"{url}/v1/chat/completions", headers=headers, json=payload) as response:
                async for chunk in response.content.iter_chunks():
                    lines = chunk[0].decode("utf-8").split('\n\n')
                    for line in lines:
                        try:
                            line = line.replace("data:", "").strip()
                            if line and line != '[DONE]':
                                line = json.loads(line)
                                if "delta" in line["choices"][0]:
                                    yield change(line)
                        except:
                            pass

    except Exception as e:
        raise ServerError("Unexpected error :(")


from setuptools import setup, find_packages

def readme():
  with open('README.md', 'r') as f:
    return f.read()

setup(
  name='mnnai',
  version='5.1.0',
  author='mkshustov',
  author_email='reverse.api.mnn@gmail.com',
  description='Module for using MNN API',
  long_description=readme(),
  long_description_content_type='text/markdown',
  url='https://github.com/mkshustov/MNNAI',
  packages=find_packages(),
  install_requires=['requests>=2.25.1', 'aiohttp'],
  classifiers=[
    'Programming Language :: Python :: 3.11',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent'
  ],
  keywords='ai MNN chatgpt mnnai mnn',
  project_urls={
    'Documentation': 'https://github.com/mkshustov/MNNAI',
    'Site': 'http://mnnai.ru'
  },
  python_requires='>=3.7'
)
